@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
@@ -18,7 +18,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
@@ -26,11 +26,17 @@ class Horde_Imap_Client_DateTime extends DateTime
 {
     /**
      */
-    public function __construct($time = null)
+    public function __construct($time = null, $tz = null)
     {
+        /* See https://bugs.php.net/bug.php?id=67118 */
+        $bug_67118 = (PHP_MINOR_VERSION >= 6) ||
+                     in_array(PHP_VERSION, array('5.4.29', '5.5.13'));
         $tz = new DateTimeZone('UTC');
 
         try {
+            if ($bug_67118) {
+                new DateTime($time, $tz);
+            }
             parent::__construct($time, $tz);
             return;
         } catch (Exception $e) {}
@@ -38,6 +44,9 @@ class Horde_Imap_Client_DateTime extends DateTime
         /* Bug #5717 - Check for UT vs. UTC. */
         if (substr(rtrim($time), -3) === ' UT') {
             try {
+                if ($bug_67118) {
+                    new DateTime($time . 'C', $tz);
+                }
                 parent::__construct($time . 'C', $tz);
                 return;
             } catch (Exception $e) {}
@@ -48,6 +57,9 @@ class Horde_Imap_Client_DateTime extends DateTime
         $date = preg_replace("/\s*\([^\)]+\)\s*$/", '', $time, -1, $i);
         if ($i) {
             try {
+                if ($bug_67118) {
+                    new DateTime($date, $tz);
+                }
                 parent::__construct($date, $tz);
                 return;
             } catch (Exception $e) {}
@@ -63,7 +75,7 @@ class Horde_Imap_Client_DateTime extends DateTime
     {
         return $this->error()
             ? '0'
-            : $this->format('U');
+            : strval($this->getTimestamp());
     }
 
     /**
@@ -73,7 +85,7 @@ class Horde_Imap_Client_DateTime extends DateTime
      */
     public function error()
     {
-        return (intval($this->format('U')) === -1);
+        return ($this->getTimestamp() === -1);
     }
 
 }
