@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2008-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2008-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2008-2013 Horde LLC
+ * @copyright 2008-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
@@ -18,21 +18,27 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2008-2013 Horde LLC
+ * @copyright 2008-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
 class Horde_Imap_Client_Search_Query implements Serializable
 {
-    /* Serialized version. */
+    /**
+     * Serialized version.
+     */
     const VERSION = 3;
 
-    /* Constants for dateSearch() */
+    /**
+     * Constants for dateSearch()
+     */
     const DATE_BEFORE = 'BEFORE';
     const DATE_ON = 'ON';
     const DATE_SINCE = 'SINCE';
 
-    /* Constants for intervalSearch() */
+    /**
+     * Constants for intervalSearch()
+     */
     const INTERVAL_OLDER = 'OLDER';
     const INTERVAL_YOUNGER = 'YOUNGER';
 
@@ -56,8 +62,12 @@ class Horde_Imap_Client_Search_Query implements Serializable
      */
     public function __toString()
     {
-        $res = $this->build(null);
-        return $res['query']->escape();
+        try {
+            $res = $this->build(null);
+            return $res['query']->escape();
+        } catch (Exception $e) {
+            return '';
+        }
     }
 
     /**
@@ -93,13 +103,16 @@ class Horde_Imap_Client_Search_Query implements Serializable
     /**
      * Builds an IMAP4rev1 compliant search string.
      *
-     * @param array $exts  The list of extensions supported by the server.
-     *                     This determines whether certain criteria can be
-     *                     used, and determines whether workarounds are used
-     *                     for other criteria. In the format returned by
-     *                     Horde_Imap_Client_Base::capability(). If this value
-     *                     is null, all extensions are assumed to be
-     *                     available.
+     * @todo  Change default of $exts to null.
+     *
+     * @param Horde_Imap_Client_Base $exts  The server object this query will
+     *                                      be run on (@since 2.24.0), a
+     *                                      Hore_Imap_Client_Data_Capability
+     *                                      object (@since 2.24.0), or the
+     *                                      list of extensions present
+     *                                      on the server (@deprecated).
+     *                                      If null, all extensions are
+     *                                      assumed to be available.
      *
      * @return array  An array with these elements:
      *   - charset: (string) The charset of the search string. If null, no
@@ -113,6 +126,21 @@ class Horde_Imap_Client_Search_Query implements Serializable
      */
     public function build($exts = array())
     {
+        /* @todo: BC */
+        if (is_array($exts)) {
+            $tmp = new Horde_Imap_Client_Data_Capability_Imap();
+            foreach ($exts as $key => $val) {
+                $tmp->add($key, is_array($val) ? $val : null);
+            }
+            $exts = $tmp;
+        } elseif (!is_null($exts)) {
+            if ($exts instanceof Horde_Imap_Client_Base) {
+                $exts = $exts->capability;
+            } elseif (!($exts instanceof Horde_Imap_Client_Data_Capability)) {
+                throw new InvalidArgumentException('Incorrect $exts parameter');
+            }
+        }
+
         $temp = array(
             'cmds' => new Horde_Imap_Client_Data_Format_List(),
             'exts' => $exts,
@@ -223,7 +251,7 @@ class Horde_Imap_Client_Search_Query implements Serializable
 
         if (isset($ptr['ids']) &&
             (count($ptr['ids']['ids']) || $ptr['ids']['ids']->special)) {
-            $this->_addFuzzy(!empty($val['fuzzy']), $temp);
+            $this->_addFuzzy(!empty($ptr['ids']['fuzzy']), $temp);
             if (!empty($ptr['ids']['not'])) {
                 $cmds->add('NOT');
             }
@@ -251,7 +279,7 @@ class Horde_Imap_Client_Search_Query implements Serializable
         }
 
         if (!empty($ptr['within'])) {
-            if (is_null($exts) || isset($exts['WITHIN'])) {
+            if (is_null($exts) || $exts->query('WITHIN')) {
                 $exts_used[] = 'WITHIN';
             }
 
@@ -261,7 +289,7 @@ class Horde_Imap_Client_Search_Query implements Serializable
                     $cmds->add('NOT');
                 }
 
-                if (is_null($exts) || isset($exts['WITHIN'])) {
+                if (is_null($exts) || $exts->query('WITHIN')) {
                     $cmds->add(array(
                         $key,
                         new Horde_Imap_Client_Data_Format_Number($val['interval'])
@@ -278,8 +306,8 @@ class Horde_Imap_Client_Search_Query implements Serializable
         }
 
         if (!empty($ptr['modseq'])) {
-            if (!is_null($exts) && !isset($exts['CONDSTORE'])) {
-                throw new Horde_Imap_Client_Exception_NoSupportExtension('IMAP Server does not support CONDSTORE.');
+            if (!is_null($exts) && !$exts->query('CONDSTORE')) {
+                throw new Horde_Imap_Client_Exception_NoSupportExtension('CONDSTORE');
             }
 
             $exts_used[] = 'CONDSTORE';
@@ -300,8 +328,8 @@ class Horde_Imap_Client_Search_Query implements Serializable
         }
 
         if (isset($ptr['prevsearch'])) {
-            if (!is_null($exts) && !isset($exts['SEARCHRES'])) {
-                throw new Horde_Imap_Client_Exception_NoSupportExtension('IMAP Server does not support SEARCHRES.');
+            if (!is_null($exts) && !$exts->query('SEARCHRES')) {
+                throw new Horde_Imap_Client_Exception_NoSupportExtension('SEARCHRES');
             }
 
             $exts_used[] = 'SEARCHRES';
@@ -374,9 +402,8 @@ class Horde_Imap_Client_Search_Query implements Serializable
     protected function _addFuzzy($add, &$temp)
     {
         if ($add) {
-            if (!isset($temp['exts']['SEARCH']) ||
-                !in_array('FUZZY', $temp['exts']['SEARCH'])) {
-                throw new Horde_Imap_Client_Exception_NoSupportExtension('IMAP Server does not support SEARCH=FUZZY.');
+            if (!$temp['exts']->query('SEARCH', 'FUZZY')) {
+                throw new Horde_Imap_Client_Exception_NoSupportExtension('SEARCH=FUZZY');
             }
             $temp['cmds']->add('FUZZY');
             $temp['exts_used'][] = 'SEARCH=FUZZY';
@@ -497,6 +524,8 @@ class Horde_Imap_Client_Search_Query implements Serializable
     /**
      * Search for messages smaller/larger than a certain size.
      *
+     * @todo: Remove $not for 3.0
+     *
      * @param integer $size    The size (in bytes).
      * @param boolean $larger  Search for messages larger than $size?
      * @param boolean $not     If true, do a 'NOT' search of $text.
@@ -518,12 +547,12 @@ class Horde_Imap_Client_Search_Query implements Serializable
     }
 
     /**
-     * Search for messages within a given ID sequence range. Only one message
-     * range can be specified per query.
+     * Search for messages within a given UID range. Only one message range
+     * can be specified per query.
      *
-     * @param Horde_Imap_Client_Ids $ids  The list of IDs to search.
+     * @param Horde_Imap_Client_Ids $ids  The list of UIDs to search.
      * @param boolean $not                If true, do a 'NOT' search of the
-     *                                    IDs.
+     *                                    UIDs.
      * @param array $opts                 Additional options:
      *   - fuzzy: (boolean) If true, perform a fuzzy search. The IMAP server
      *            MUST support RFC 6203.
@@ -650,7 +679,7 @@ class Horde_Imap_Client_Search_Query implements Serializable
 
     /**
      * Search for messages modified since a specific moment. The IMAP server
-     * must support the CONDSTORE extension (RFC 4551) for this query to be
+     * must support the CONDSTORE extension (RFC 7162) for this query to be
      * used.
      *
      * @param integer $value  The mod-sequence value.
