@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
@@ -20,7 +20,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @internal
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
@@ -125,13 +125,13 @@ class Horde_Imap_Client_Socket_ClientSort
         /* Step 1: Sort by base subject (already done).
          * Step 2: Sort by sent date within each thread. */
         foreach (array_keys($sorted) as $key) {
-            asort($sorted[$key], SORT_NUMERIC);
+            $this->_stableAsort($sorted[$key]);
             $tsort[$key] = reset($sorted[$key]);
         }
 
         /* Step 3: Sort by the sent date of the first message in the
          * thread. */
-        asort($tsort, SORT_NUMERIC);
+        $this->_stableAsort($tsort);
 
         /* Now, $tsort contains the order of the threads, and each thread
          * is sorted in $sorted. */
@@ -142,7 +142,10 @@ class Horde_Imap_Client_Socket_ClientSort
             ) + array_fill_keys(array_slice($keys, 1) , 1);
         }
 
-        return new Horde_Imap_Client_Data_Thread($out, $uids ? 'uid' : 'sequence');
+        return new Horde_Imap_Client_Data_Thread(
+            $out,
+            $uids ? 'uid' : 'sequence'
+        );
     }
 
     /**
@@ -314,4 +317,20 @@ class Horde_Imap_Client_Socket_ClientSort
         return $dates;
     }
 
+    /**
+     * Stable asort() function.
+     *
+     * PHP's asort() (BWT) is not a stable sort - identical values have no
+     * guarantee of key order. Use Schwartzian Transform instead. See:
+     * http://notmysock.org/blog/php/schwartzian-transform.html
+     *
+     * @param array &$a  Array to sort.
+     */
+    protected function _stableAsort(&$a)
+    {
+        array_walk($a, function(&$v, $k) { $v = array($v, $k); });
+        asort($a);
+        array_walk($a, function(&$v, $k) { $v = $v[0]; });
+    }
+
 }
