@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -43,7 +43,7 @@
  *
  * @category  Horde
  * @copyright 2002 Richard Heyes
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
@@ -55,6 +55,7 @@
  * IMAP equivalents.
  *
  * This driver implements the following POP3-related RFCs:
+ * <pre>
  *   - STD 53/RFC 1939: POP3 specification
  *   - RFC 2195: CRAM-MD5 authentication
  *   - RFC 2449: POP3 extension mechanism
@@ -63,17 +64,22 @@
  *   - RFC 3206: AUTH/SYS response codes
  *   - RFC 4616: AUTH=PLAIN
  *   - RFC 5034: POP3 SASL
+ * </pre>
  *
  * @author    Richard Heyes <richard@phpguru.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
  * @copyright 2002 Richard Heyes
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Imap_Client
  */
 class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
 {
+    /* Internal key used to store mailbox level cache data. \1 is not a valid
+     * ID in POP3, so it should be safe to use. */
+    const MBOX_CACHE = "\1mbox";
+
     /**
      * The default ports to use for a connection.
      *
@@ -95,12 +101,27 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      */
     protected $_fetchDataClass = 'Horde_Imap_Client_Data_Fetch_Pop3';
 
+    /**
+     */
+    public function __get($name)
+    {
+        $out = parent::__get($name);
+
+        switch ($name) {
+        case 'url':
+            $url->protocol = 'pop3';
+            break;
+        }
+
+        return $out;
+    }
+
     /**
      */
     protected function _initCache($current = false)
     {
         return parent::_initCache($current) &&
-               $this->queryCapability('UIDL');
+               $this->_capability('UIDL');
     }
 
     /**
@@ -112,11 +133,11 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
 
     /**
      */
-    protected function _capability()
+    protected function _initCapability()
     {
         $this->_connect();
 
-        $capability = array();
+        $c = new Horde_Imap_Client_Data_Capability();
 
         try {
             $res = $this->_sendLine('CAPA', array(
@@ -125,33 +146,31 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
 
             foreach ($res['data'] as $val) {
                 $prefix = explode(' ', $val);
-                $capability[strtoupper($prefix[0])] = (count($prefix) > 1)
-                    ? array_slice($prefix, 1)
-                    : true;
+                $c->add($prefix[0], array_slice($prefix, 1));
             }
         } catch (Horde_Imap_Client_Exception $e) {
             $this->_temp['no_capa'] = true;
 
             /* Need to probe for capabilities if CAPA command is not
              * available. */
-            $capability = array('USER' => true);
+            $c->add('USER');
 
             /* Capability sniffing only guaranteed after authentication is
              * completed (if any). */
             if (!empty($this->_init['authmethod'])) {
                 $this->_pop3Cache('uidl');
                 if (empty($this->_temp['no_uidl'])) {
-                    $capability['UIDL'] = true;
+                    $c->add('UIDL');
                 }
 
                 $this->_pop3Cache('top', 1);
                 if (empty($this->_temp['no_top'])) {
-                    $capability['TOP'] = true;
+                    $c->add('TOP');
                 }
             }
         }
 
-        $this->_setInit('capability', $capability);
+        $this->_setInit('capability', $c);
     }
 
     /**
@@ -180,46 +199,60 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      */
     protected function _login()
     {
-        $this->_connect();
-
-        $secure = $this->getParam('secure');
-
-        // Switch to secure channel if using TLS.
-        if (!$this->isSecureConnection() &&
-            (($secure === 'tls') || $secure === true)) {
-            // Switch over to a TLS connection.
-            if (!$this->queryCapability('STLS')) {
-                if ($secure === 'tls') {
-                    throw new Horde_Imap_Client_Exception(
-                        Horde_Imap_Client_Translation::t("Could not open secure connection to the POP3 server.") . ' ' . Horde_Imap_Client_Translation::t("Server does not support secure connections."),
-                        Horde_Imap_Client_Exception::LOGIN_TLSFAILURE
-                    );
-                } else {
-                    $this->setParam('secure', false);
-                }
-            } else {
-                $this->_sendLine('STLS');
-
-                $this->setParam('secure', 'tls');
-
-                if (!$this->_connection->startTls()) {
-                    $this->logout();
-                    throw new Horde_Imap_Client_Exception(
-                        Horde_Imap_Client_Translation::t("Could not open secure connection to the POP3 server."),
-                        Horde_Imap_Client_Exception::LOGIN_TLSFAILURE
-                    );
-                }
-                $this->_debug->info('Successfully completed TLS negotiation.');
-            }
-
-            // Expire cached CAPABILITY information
-            $this->_setInit('capability');
+        /* Blank passwords are not allowed, so no need to even try
+         * authentication to determine this. */
+        if (!strlen($this->getParam('password'))) {
+            throw new Horde_Imap_Client_Exception(
+                Horde_Imap_Client_Translation::r("No password provided."),
+                Horde_Imap_Client_Exception::LOGIN_AUTHENTICATIONFAILED
+            );
         }
 
-        if (empty($this->_init['authmethod'])) {
-            $auth_mech = ($sasl = $this->queryCapability('SASL'))
-                ? $sasl
-                : array();
+        $this->_connect();
+
+        $first_login = empty($this->_init['authmethod']);
+
+        // Switch to secure channel if using TLS.
+        if (!$this->isSecureConnection()) {
+            $secure = $this->getParam('secure');
+
+            if (($secure === 'tls') || $secure === true) {
+                // Switch over to a TLS connection.
+                if ($first_login && !$this->_capability('STLS')) {
+                    if ($secure === 'tls') {
+                        throw new Horde_Imap_Client_Exception(
+                            Horde_Imap_Client_Translation::r("Could not open secure connection to the POP3 server.") . ' ' . Horde_Imap_Client_Translation::r("Server does not support secure connections."),
+                            Horde_Imap_Client_Exception::LOGIN_TLSFAILURE
+                        );
+                    } else {
+                        $this->setParam('secure', false);
+                    }
+                } else {
+                    $this->_sendLine('STLS');
+
+                    $this->setParam('secure', 'tls');
+
+                    if (!$this->_connection->startTls()) {
+                        $this->logout();
+                        throw new Horde_Imap_Client_Exception(
+                            Horde_Imap_Client_Translation::r("Could not open secure connection to the POP3 server."),
+                            Horde_Imap_Client_Exception::LOGIN_TLSFAILURE
+                        );
+                    }
+                    $this->_debug->info('Successfully completed TLS negotiation.');
+                }
+
+                // Expire cached CAPABILITY information
+                $this->_setInit('capability');
+            } else {
+                $this->setParam('secure', false);
+            }
+        }
+
+        if ($first_login) {
+            /* At least one server (Dovecot 1.x) may return SASL capability
+             * with no arguments. */
+            $auth_mech = $this->_capability()->getParams('SASL');
 
             if (isset($this->_temp['pop3timestamp'])) {
                 $auth_mech[] = 'APOP';
@@ -236,13 +269,15 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
                 $this->_setInit('authmethod', $method);
 
                 if (!empty($this->_temp['no_capa']) ||
-                    !$this->queryCapability('UIDL')) {
-                    $this->_capability();
+                    !$this->_capability('UIDL')) {
+                    $this->_setInit('capability');
                 }
 
                 return true;
             } catch (Horde_Imap_Client_Exception $e) {
-                if (!empty($this->_init['authmethod'])) {
+                if (!empty($this->_init['authmethod']) &&
+                    ($e->getCode() != $e::LOGIN_UNAVAILABLE) &&
+                    ($e->getCode() != $e::POP3_TEMP_ERROR)) {
                     $this->_setInit();
                     return $this->login();
                 }
@@ -250,8 +285,8 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
         }
 
         throw new Horde_Imap_Client_Exception(
-            Horde_Imap_Client_Translation::t("POP3 server denied authentication."),
-            $e->getCode() ? $e->getCode() : Horde_Imap_Client_Exception::LOGIN_AUTHENTICATIONFAILED
+            Horde_Imap_Client_Translation::r("POP3 server denied authentication."),
+            $e->getCode() ?: $e::LOGIN_AUTHENTICATIONFAILED
         );
     }
 
@@ -276,12 +311,9 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
                     'debug' => $this->_debug
                 )
             );
-            if (!$this->_connection->secure) {
-                $this->setParam('secure', false);
-            }
         } catch (Horde\Socket\Client\Exception $e) {
             $e2 = new Horde_Imap_Client_Exception(
-                Horde_Imap_Client_Translation::t("Error connecting to mail server."),
+                Horde_Imap_Client_Translation::r("Error connecting to mail server."),
                 Horde_Imap_Client_Exception::SERVER_CONNECT
             );
             $e2->details = $e->details;
@@ -317,7 +349,7 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
             $challenge = $this->_sendLine('AUTH ' . $method);
             $response = base64_encode($username . ' ' . hash_hmac(strtolower(substr($method, 5)), base64_decode(substr($challenge['resp'], 2)), $password, true));
             $this->_sendLine($response, array(
-                'debug' => sprintf('[%s Response - username: %s]', $method, $username)
+                'debug' => sprintf('[AUTH Response (username: %s)]', $username)
             ));
             break;
 
@@ -332,11 +364,11 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
                 'pop3'
             ));
             $sresponse = $this->_sendLine($response, array(
-                'debug' => sprintf('[%s Response - username: %s]', $method, $username)
+                'debug' => sprintf('[AUTH Response (username: %s)]', $username)
             ));
             if (stripos(base64_decode(substr($sresponse['resp'], 2)), 'rspauth=') === false) {
                 throw new Horde_Imap_Client_Exception(
-                    Horde_Imap_Client_Translation::t("Unexpected response from server when authenticating."),
+                    Horde_Imap_Client_Translation::r("Unexpected response from server when authenticating."),
                     Horde_Imap_Client_Exception::SERVER_CONNECT
                 );
             }
@@ -348,11 +380,9 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
         case 'LOGIN':
             // RFC 4616 (AUTH=PLAIN) & 5034 (POP3 SASL)
             $this->_sendLine('AUTH LOGIN');
-            $this->_sendLine(base64_encode($username), array(
-                'debug' => sprintf('[AUTH LOGIN Command - username: %s]', $username)
-            ));
+            $this->_sendLine(base64_encode($username));
             $this->_sendLine(base64_encode($password), array(
-                'debug' => '[AUTH LOGIN Command - password]'
+                'debug' => sprintf('[AUTH Password (username: %s)]', $username)
             ));
             break;
 
@@ -363,7 +393,7 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
                 $username,
                 $password
             ))), array(
-                'debug' => sprintf('[AUTH PLAIN Command - username: %s]', $username)
+                'debug' => sprintf('AUTH PLAIN [Auth Response (username: %s)]', $username)
             ));
             break;
 
@@ -376,15 +406,17 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
             // RFC 1939 [7]
             $this->_sendLine('USER ' . $username);
             $this->_sendLine('PASS ' . $password, array(
-                'debug' => '[USER Command - password]'
+                'debug' => 'PASS [Password]'
             ));
             break;
 
         default:
-            throw new Horde_Imap_Client_Exception(
-                sprintf(Horde_Imap_Client_Translation::t("Unknown authentication method: %s"), $method),
+            $e = new Horde_Imap_Client_Exception(
+                Horde_Imap_Client_Translation::r("Unknown authentication method: %s"),
                 Horde_Imap_Client_Exception::SERVER_CONNECT
             );
+            $e->messagePrintf(array($method));
+            throw $e;
         }
     }
 
@@ -411,10 +443,9 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      */
     protected function _getID()
     {
-        $id = $this->queryCapability('IMPLEMENTATION');
-        return empty($id)
-            ? array()
-            : array('implementation' => $id);
+        return ($id = $this->_capability()->getParams('IMPLEMENTATION'))
+            ? array('implementation' => reset($id))
+            : array();
     }
 
     /**
@@ -482,18 +513,13 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      */
     protected function _listMailboxes($pattern, $mode, $options)
     {
-        $tmp = array(
-            'mailbox' => Horde_Imap_Client_Mailbox::get('INBOX')
+        return array(
+            'INBOX' => array(
+                'attributes' => array(),
+                'delimiter' => '',
+                'mailbox' => Horde_Imap_Client_Mailbox::get('INBOX')
+            )
         );
-
-        if (!empty($options['attributes'])) {
-            $tmp['attributes'] = array();
-        }
-        if (!empty($options['delimiter'])) {
-            $tmp['delimiter'] = '';
-        }
-
-        return array('INBOX' => $tmp);
     }
 
     /**
@@ -523,13 +549,25 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
 
         // No need for STATUS_UIDNEXT_FORCE handling since STATUS_UIDNEXT will
         // always return a value.
+        $uidl = $this->_capability('UIDL');
         if ($flags & Horde_Imap_Client::STATUS_UIDNEXT) {
-            $res = $this->_pop3Cache('stat');
-            $ret['uidnext'] = $res['msgs'] + 1;
+            if ($uidl) {
+                $res = $this->_pop3Cache('uidl');
+                $res = end($res);
+                /* Increment the last character.  If it is 0x7f, then
+                 * add another character to UID. */
+                $ord = ord(substr($res, -1));
+                $ret['uidnext'] = (++$ord > 126)
+                    ? ($res . '!')
+                    : substr_replace($res, chr($ord), -1);
+            } else {
+                $res = $this->_pop3Cache('stat');
+                $ret['uidnext'] = $res['msgs'] + 1;
+            }
         }
 
         if ($flags & Horde_Imap_Client::STATUS_UIDVALIDITY) {
-            $ret['uidvalidity'] = $this->queryCapability('UIDL')
+            $ret['uidvalidity'] = $uidl
                 ? 1
                 : microtime(true);
         }
@@ -608,6 +646,12 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
             $res = $tmp;
         }
 
+        if (!empty($options['partial'])) {
+            $partial = $this->getIdsOb($options['partial'], true);
+            $min = $partial->min - 1;
+            $res = array_slice($res, $min, $partial->max - $min);
+        }
+
         $ret = array();
         foreach ($options['results'] as $val) {
             switch ($val) {
@@ -851,7 +895,9 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      *
      * @throws Horde_Imap_Client_Exception
      */
-    protected function _pop3Cache($type, $index = null, $data = null)
+    protected function _pop3Cache(
+        $type, $index = self::MBOX_CACHE, $data = null
+    )
     {
         if (isset($this->_temp['pop3cache'][$index][$type])) {
             if ($type == 'msg') {
@@ -864,7 +910,7 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
         case 'hdr':
         case 'top':
             $data = null;
-            if ($this->queryCapability('TOP') || ($type == 'top')) {
+            if (($type == 'top') || $this->_capability('TOP')) {
                 try {
                     $res = $this->_sendLine('TOP ' . $index . ' 0', array(
                         'multiline' => 'stream'
@@ -989,6 +1035,11 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
                 try {
                     $this->_sendLine('DELE ' . $id);
                     $this->_deleted[] = $id;
+
+                    unset(
+                        $this->_temp['pop3cache'][self::MBOX_CACHE],
+                        $this->_temp['pop3cache'][$id]
+                    );
                 } catch (Horde_Imap_Client_Exception $e) {}
             }
         }
@@ -1135,24 +1186,33 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
      */
     protected function _sendLine($cmd, $options = array())
     {
-        $old_debug = $this->_debug->debug;
         if (!empty($options['debug'])) {
-            $this->_debug->raw($options['debug'] . "\n");
-            $this->_debug->debug = false;
+            $this->_debug->client($options['debug']);
+        }
+
+        if ($this->_debug->debug) {
+            $timer = new Horde_Support_Timer();
+            $timer->push();
         }
 
         try {
-            $this->_connection->write($cmd);
+            $this->_connection->write($cmd, empty($options['debug']));
         } catch (Horde_Imap_Client_Exception $e) {
-            $this->_debug->debug = $old_debug;
             throw $e;
         }
 
-        $this->_debug->debug = $old_debug;
-
-        return $this->_getResponse(
+        $resp = $this->_getResponse(
             empty($options['multiline']) ? false : $options['multiline']
         );
+
+        if ($this->_debug->debug) {
+            $this->_debug->info(sprintf(
+                'Command took %s seconds.',
+                round($timer->pop(), 4)
+            ));
+        }
+
+        return $resp;
     }
 
     /**
@@ -1172,9 +1232,9 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
 
         $read = explode(' ', rtrim($this->_connection->read(), "\r\n"), 2);
         if (!in_array($read[0], array('+OK', '-ERR', '+'))) {
-            $this->_debug->info("ERROR: IMAP read/timeout error.");
+            $this->_debug->info('ERROR: IMAP read/timeout error.');
             throw new Horde_Imap_Client_Exception(
-                Horde_Imap_Client_Translation::t("Error when communicating with the mail server."),
+                Horde_Imap_Client_Translation::r("Error when communicating with the mail server."),
                 Horde_Imap_Client_Exception::SERVER_READERROR
             );
         }
@@ -1182,7 +1242,7 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
         $respcode = null;
         if (isset($read[1]) &&
             isset($this->_init['capability']) &&
-            $this->queryCapability('RESP-CODES')) {
+            $this->_capability('RESP-CODES')) {
             $respcode = $this->_parseResponseCode($read[1]);
         }
 
@@ -1233,7 +1293,7 @@ class Horde_Imap_Client_Socket_Pop3 extends Horde_Imap_Client_Base
             }
 
             $e = new Horde_Imap_Client_Exception(
-                Horde_Imap_Client_Translation::t("POP3 error reported by server."),
+                Horde_Imap_Client_Translation::r("POP3 error reported by server."),
                 $errcode
             );
             $e->details = $errtext;
