@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2011-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category   Horde
- * @copyright  2011-2013 Horde LLC
+ * @copyright  2011-2014 Horde LLC
  * @license    http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package    Imap_Client
  * @subpackage UnitTests
@@ -17,7 +17,7 @@
  *
  * @author     Michael Slusarz <slusarz@horde.org>
  * @category   Horde
- * @copyright  2011-2013 Horde LLC
+ * @copyright  2011-2014 Horde LLC
  * @ignore
  * @license    http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package    Imap_Client
@@ -42,6 +42,22 @@ class Horde_Imap_Client_IdsTest extends PHPUnit_Framework_TestCase
         $this->assertFalse($ids->isEmpty());
     }
 
+    public function testIgnoreNullInput()
+    {
+        $ids = new Horde_Imap_Client_Ids(null);
+        $this->assertEquals(
+            0,
+            count($ids)
+        );
+
+        $ids = new Horde_Imap_Client_Ids();
+        $ids->add(null);
+        $this->assertEquals(
+            0,
+            count($ids)
+        );
+    }
+
     public function testEmptyIdsArray()
     {
         $ids = new Horde_Imap_Client_Ids(array());
@@ -179,40 +195,6 @@ class Horde_Imap_Client_IdsTest extends PHPUnit_Framework_TestCase
         );
     }
 
-    public function testPop3SequenceStringGenerate()
-    {
-        $this->assertEquals(
-            'ABCDEFGHIJ ABCDE',
-            strval(new Horde_Imap_Client_Ids_Pop3(array('ABCDEFGHIJ', 'ABCDE')))
-        );
-
-        $this->assertEquals(
-            'ABCDEFGHIJ',
-            strval(new Horde_Imap_Client_Ids_Pop3('ABCDEFGHIJ'))
-        );
-    }
-
-    public function testPop3SequenceStringParse()
-    {
-        $ids = new Horde_Imap_Client_Ids_Pop3('ABCDEFGHIJ ABCDE');
-        $this->assertEquals(
-            array('ABCDEFGHIJ', 'ABCDE'),
-            $ids->ids
-        );
-
-        $ids = new Horde_Imap_Client_Ids_Pop3('ABCDEFGHIJ ABC ABCDE');
-        $this->assertEquals(
-            array('ABCDEFGHIJ', 'ABC', 'ABCDE'),
-            $ids->ids
-        );
-
-        $ids = new Horde_Imap_Client_Ids_Pop3('10:12');
-        $this->assertEquals(
-            array('10:12'),
-            $ids->ids
-        );
-    }
-
     public function testSplit()
     {
         // ~5000 non-sequential IDs
@@ -295,4 +277,40 @@ class Horde_Imap_Client_IdsTest extends PHPUnit_Framework_TestCase
         );
     }
 
+    public function testMinAndMax()
+    {
+        $ids = new Horde_Imap_Client_Ids(array(1));
+
+        $this->assertEquals(
+            1,
+            $ids->min
+        );
+        $this->assertEquals(
+            1,
+            $ids->max
+        );
+
+        $ids2 = new Horde_Imap_Client_Ids(array(1, 2));
+
+        $this->assertEquals(
+            1,
+            $ids2->min
+        );
+        $this->assertEquals(
+            2,
+            $ids2->max
+        );
+
+        $ids3 = new Horde_Imap_Client_Ids(array(1, 5, 3));
+
+        $this->assertEquals(
+            1,
+            $ids3->min
+        );
+        $this->assertEquals(
+            5,
+            $ids3->max
+        );
+    }
+
 }
